@@ -34,15 +34,22 @@ The development version can be found in the repository at
 
 A very brief overview of the language contents follows:
 
-* Cells and Euclidean spaces::
+* Euclidean spaces::
 
-    interval, triangle, tetrahedron, quadrilateral, hexahedron,
+    Space,
     R1, R2, R3,
-    Cell, Space
+
+* Cells::
+
+    Cell,
+    interval, triangle, tetrahedron,
+    quadrilateral, hexahedron,
+    cell1D, cell2D, cell3D,
 
 * Elements::
 
-    FiniteElement, MixedElement, VectorElement, TensorElement
+    FiniteElement,
+    MixedElement, VectorElement, TensorElement
     EnrichedElement, RestrictedElement
 
 * Arguments::
@@ -59,11 +66,12 @@ A very brief overview of the language contents follows:
 
 * Literal constants::
 
-    Identity
+    Identity, PermutationSymbol
 
 * Geometric quantities::
 
-    SpatialCoordinate, FacetNormal
+    SpatialCoordinate, FacetNormal, Circumradius,
+    CellVolume, CellSurfaceArea, FacetArea
 
 * Indices::
 
@@ -82,29 +90,36 @@ A very brief overview of the language contents follows:
 * Tensor algebra operators::
 
     outer, inner, dot, cross,
-    transpose, tr,
     det, inv, cofac,
-    dev, skew, sym,
+    transpose, tr, diag, diag_vector,
+    dev, skew, sym
+
+* Elementwise tensor operators::
+
+    elem_mult, elem_div, elem_pow, elem_op
 
 * Differential operators::
 
     variable, diff,
-    Dx, grad, div, curl, rot, Dn
+    grad, div, nabla_grad, nabla_div,
+    Dx, Dn, curl, rot
 
 * Nonlinear functions::
 
-    abs, sign, sqrt, exp, ln, cos, sin, tan, acos, asin, atan
+    abs, sign, sqrt, exp, ln, erf,
+    cos, sin, tan, acos, asin, atan,
+    bessel_J, bessel_Y, bessel_I, bessel_K
 
 * Discontinuous Galerkin operators:
     jump, avg, v('+'), v('-')
 
 * Conditional operators::
 
-    eq, ne, le, ge, lt, gt, conditional,
+    eq, ne, le, ge, lt, gt, And, Or, Not, conditional
 
 * Integral measures::
 
-    dx, ds, dS, dE
+    dx, ds, dS, dE, dc
 
 * Form transformations::
 
@@ -124,19 +139,18 @@ A very brief overview of the language contents follows:
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Kristian Oelgaard, 2009
+# Modified by Kristian B. Oelgaard, 2009, 2011
 # Modified by Anders Logg, 2009.
 #
-# First added:  2011-04-08
-# Last changed: 2010-02-01
+# Last changed: 2011-12-07
 
-__version__ = "0.9.1"
+__version__ = "1.0.0"
 
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
@@ -154,8 +168,8 @@ from ufl.log import get_handler, get_logger, set_handler, set_level, add_logfile
     UFLException, DEBUG, INFO, WARNING, ERROR, CRITICAL
 
 # Types for geometric quantities
-from ufl.geometry import Cell, Space, SpatialCoordinate, FacetNormal, CellVolume, \
-                         Circumradius
+from ufl.geometry import Cell, Space, SpatialCoordinate, FacetNormal, Circumradius, \
+                         CellVolume, CellSurfaceArea, FacetArea
 
 # Finite elements classes
 from ufl.finiteelement import FiniteElementBase, FiniteElement, \
@@ -176,7 +190,7 @@ from ufl.coefficient import Coefficient, Coefficients, \
 from ufl.split_functions import split
 
 # Literal constants
-from ufl.constantvalue import PermutationSymbol, Identity, zero, Zero, as_ufl # TODO: Remove Zero
+from ufl.constantvalue import PermutationSymbol, Identity, zero, as_ufl
 
 # Indexing of tensor expressions
 from ufl.indexing import Index, indices
@@ -193,12 +207,17 @@ from ufl.tensors import unit_vector, unit_vectors, unit_matrix, unit_matrices
 from ufl.operators import rank, shape, \
                        outer, inner, dot, cross, \
                        det, inv, cofac, \
-                       transpose, tr, dev, skew, sym, \
-                       sqrt, exp, ln, cos, sin, tan, acos, asin, atan, \
-                       eq, ne, le, ge, lt, gt, conditional, sign, \
+                       transpose, tr, diag, diag_vector, \
+                       dev, skew, sym, \
+                       sqrt, exp, ln, erf, \
+                       cos, sin, tan, acos, asin, atan, \
+                       bessel_J, bessel_Y, bessel_I, bessel_K, \
+                       eq, ne, le, ge, lt, gt, And, Or, Not, \
+                       conditional, sign, \
                        variable, diff, \
-                       Dx,  grad, div, curl, rot, Dn, exterior_derivative,\
-                       jump, avg
+                       Dx,  grad, div, curl, rot, nabla_grad, nabla_div, Dn, exterior_derivative, \
+                       jump, avg, \
+                       elem_mult, elem_div, elem_pow, elem_op
 
 # Lifting
 from ufl.lifting import LiftingFunction, LiftingOperator
@@ -210,10 +229,12 @@ from ufl.form import Form
 from ufl.integral import Integral, Measure, register_domain_type
 
 # Representations of transformed forms
-from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs, system, functional, adjoint, sensitivity_rhs #, dirichlet_functional
+from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs,\
+    system, functional, adjoint, sensitivity_rhs #, dirichlet_functional
 
 # Predefined convenience objects
-from ufl.objects import vertex, interval, triangle, tetrahedron, \
+from ufl.objects import \
+    vertex, interval, triangle, tetrahedron, \
     quadrilateral, hexahedron, facet, cell1D, cell2D, cell3D, \
     i, j, k, l, p, q, r, s, \
     R1, R2, R3, \
@@ -221,3 +242,48 @@ from ufl.objects import vertex, interval, triangle, tetrahedron, \
 
 # Useful constants
 from math import e, pi
+
+__all__ = [
+    'product',
+    'get_handler', 'get_logger', 'set_handler', 'set_level', 'add_logfile',
+    'UFLException', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL',
+    'Cell', 'Space', 'SpatialCoordinate', 'FacetNormal', 'CellVolume',
+    'Circumradius', 'CellSurfaceArea', 'FacetArea',
+    'FiniteElementBase', 'FiniteElement',
+    'MixedElement', 'VectorElement', 'TensorElement', 'EnrichedElement', 'RestrictedElement',
+    'register_element', 'show_elements',
+    'Argument', 'TestFunction', 'TrialFunction',
+    'Arguments', 'TestFunctions', 'TrialFunctions',
+    'Coefficient', 'Coefficients',
+    'Constant', 'VectorConstant', 'TensorConstant',
+    'split',
+    'PermutationSymbol', 'Identity', 'zero', 'as_ufl',
+    'Index', 'indices',
+    'as_tensor', 'as_vector', 'as_matrix', 'relabel',
+    'unit_vector', 'unit_vectors', 'unit_matrix', 'unit_matrices',
+    'rank', 'shape',
+    'outer', 'inner', 'dot', 'cross',
+    'det', 'inv', 'cofac',
+    'transpose', 'tr', 'diag', 'diag_vector', 'dev', 'skew', 'sym',
+    'sqrt', 'exp', 'ln', 'erf',
+    'cos', 'sin', 'tan', 'acos', 'asin', 'atan',
+    'bessel_J', 'bessel_Y', 'bessel_I', 'bessel_K',
+    'eq', 'ne', 'le', 'ge', 'lt', 'gt', 'And', 'Or', 'Not',
+    'conditional', 'sign',
+    'variable', 'diff',
+    'Dx', 'grad', 'div', 'curl', 'rot', 'nabla_grad', 'nabla_div', 'Dn', 'exterior_derivative',
+    'jump', 'avg',
+    'elem_mult', 'elem_div', 'elem_pow', 'elem_op',
+    'LiftingFunction', 'LiftingOperator',
+    'Form',
+    'Integral', 'Measure', 'register_domain_type',
+    'replace', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs',
+    'system', 'functional', 'adjoint', 'sensitivity_rhs',
+    'R1', 'R2', 'R3',
+    'dx', 'ds', 'dS', 'dE', 'dc',
+    'vertex', 'interval', 'triangle', 'tetrahedron',
+    'quadrilateral', 'hexahedron', 'facet',
+    'cell1D', 'cell2D', 'cell3D',
+    'i', 'j', 'k', 'l', 'p', 'q', 'r', 's',
+    'e', 'pi',
+    ]
