@@ -11,19 +11,19 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
 #
 # First added:  2008-08-14
-# Last changed: 2009-05-14
+# Last changed: 2011-10-11
 
 # Function for preprocessing a form
-from ufl.algorithms.preprocess import preprocess, extract_common_cell
+from ufl.algorithms.preprocess import preprocess, preprocess_expression, extract_common_cell
 
 # Class for simple extraction of form meta data
 from ufl.algorithms.formdata import FormData
@@ -49,7 +49,7 @@ from ufl.algorithms.predicates import is_multilinear
 from ufl.algorithms.checks import validate_form
 
 # Utilites for modifying expressions and forms
-from ufl.algorithms.transformations import transform, Transformer, ReuseTransformer, apply_transformer, \
+from ufl.algorithms.transformations import transform, MultiFunction, Transformer, ReuseTransformer, apply_transformer, \
                                            ufl2ufl, ufl2uflcopy, \
                                            replace, flatten, strip_variables, \
                                            expand_compounds, \
