@@ -11,16 +11,16 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009.
 #
 # First added:  2008-12-28
-# Last changed: 2009-12-08
+# Last changed: 2011-10-25
 
 from itertools import izip
 from ufl.log import debug, error
@@ -62,17 +62,18 @@ def apply_ad(e, ad_routine):
 def expand_derivatives(form, dim=None):
     """Expand all derivatives of expr.
 
-    NB! This functionality is not finished!
-
     In the returned expression g which is mathematically
     equivalent to expr, there are no VariableDerivative
     or CoefficientDerivative objects left, and SpatialDerivative
     objects have been propagated to Terminal nodes."""
 
     # Find geometric dimension
+    cell = form.cell()
+    gdim = None if (cell is None or cell.is_undefined()) else cell.geometric_dimension()
     if dim is None:
-        cell = form.cell()
-        dim = None if cell.is_undefined() else cell.geometric_dimension()
+        dim = gdim
+    if gdim is not None:
+        ufl_assert(dim == gdim, "Expecting dim to match the geometric dimension, got dim=%r and gdim=%r." % (dim, gdim))
 
     def ad_routine(e):
         # TODO: How to switch between forward and reverse mode? Can we pick the
