@@ -13,16 +13,16 @@ indices only."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009.
 #
 # First added:  2008-04-19
-# Last changed: 2009-04-19
+# Last changed: 2011-06-08
 
 from itertools import izip
 
@@ -61,21 +61,23 @@ class IndexExpander(ReuseTransformer):
         return x
 
     def form_argument(self, x):
-        if x.shape():
-            # Get symmetry mapping if any
+        sh = x.shape()
+        if sh == ():
+            return x
+        else:
             e = x.element()
-            s = None
-            if isinstance(e, TensorElement):
-                s = e.symmetry()
-            if s is None:
-                s = {}
-            # Map component throught the symmetry mapping
+            r = len(sh)
+
+            # Get component
             c = self.component()
-            ufl_assert(len(x.shape()) == len(c), "Component size mismatch.")
-            c2 = s.get(c, c)
-            ufl_assert(len(c) == len(c2), "Component size mismatch after symmetry mapping.")
-            return x[c2]
-        return x
+            ufl_assert(r == len(c), "Component size mismatch.")
+
+            # Map it through an eventual symmetry mapping
+            s = e.symmetry()
+            c = s.get(c, c)
+            ufl_assert(r == len(c), "Component size mismatch after symmetry mapping.")
+
+            return x[c]
 
     def zero(self, x):
         ufl_assert(len(x.shape()) == len(self.component()), "Component size mismatch.")
