@@ -12,18 +12,18 @@ complete Forms into new related Forms."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
 # Modified by Garth N. Wells, 2010.
 # Modified by Marie E. Rognes, 2010.
 #
 # First added:  2008-10-01
-# Last changed: 2011-04-26
+# Last changed: 2011-08-23
 
 from itertools import izip
 
@@ -303,7 +303,10 @@ def compute_form_with_arity(form, arity):
         warning("Form has no parts with arity %d." % arity)
         return 0*form
 
-    # FIXME: Should be permutations of arguments of length arity
+    # Assuming that the form is not a sum of terms
+    # that depend on different arguments, e.g. (u+v)*dx
+    # would result in just v*dx. But that doesn't make
+    # any sense anyway.
     sub_arguments = set(arguments[:arity])
     pe = PartExtracter(sub_arguments)
     def _transform(e):
@@ -408,15 +411,28 @@ def compute_energy_norm(form, coefficient):
             "coefficient in an incompatible element space.")
     return replace(form, { u: coefficient, v: coefficient })
 
-def compute_form_adjoint(form):
+def compute_form_adjoint(form, reordered_arguments=None):
     """Compute the adjoint of a bilinear form.
 
-    This works simply by swapping the first and last arguments.
+    This works simply by changing the ordering (count) of the two arguments.
     """
     arguments = extract_arguments(form)
     ufl_assert(len(arguments) == 2, "Expecting bilinear form.")
+
     v, u = arguments
-    return replace(form, {v: u, u: v})
+    ufl_assert(v.count() < u.count(), "Mistaken assumption in code!")
+
+    if reordered_arguments is None:
+        reordered_arguments = (Argument(u.element()), Argument(v.element()))
+    reordered_u, reordered_v = reordered_arguments
+    ufl_assert(reordered_u.count() < reordered_v.count(),
+               "Ordering of new arguments is the same as the old arguments!")
+    ufl_assert(reordered_u.element() == u.element(),
+               "Element mismatch between new and old arguments (trial functions).")
+    ufl_assert(reordered_v.element() == v.element(),
+               "Element mismatch between new and old arguments (test functions).")
+
+    return replace(form, {v: reordered_v, u: reordered_u})
 
 #def compute_dirichlet_functional(form):
 #    """Compute the Dirichlet functional of a form:
