@@ -11,18 +11,20 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009.
 # Modified by Garth N. Wells, 2010.
+# Modified by Kristian B. Oelgaard, 2011
 #
 # First added:  2008-08-19
-# Last changed: 2010-06-07
+# Last changed: 2011-10-24
 
+from math import pi
 from ufl.log import error, warning, debug
 from ufl.assertions import ufl_assert
 from ufl.common import unzip, subdict, lstr
@@ -40,7 +42,7 @@ from ufl.tensors import ListTensor, ComponentTensor, as_tensor, as_scalar
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.tensoralgebra import Transposed, Outer, Inner, Dot, Cross, Trace, \
     Determinant, Inverse, Deviatoric, Cofactor
-from ufl.mathfunctions import MathFunction, Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan
+from ufl.mathfunctions import MathFunction, Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
 from ufl.restriction import Restricted, PositiveRestricted, NegativeRestricted
 from ufl.differentiation import Derivative, CoefficientDerivative,\
     SpatialDerivative, VariableDerivative
@@ -48,14 +50,14 @@ from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 
 # Lists of all Expr classes
 from ufl.classes import terminal_classes
+
 from ufl.operators import dot, inner, outer, lt, eq, conditional, sign
-from ufl.operators import sqrt, exp, ln, cos, sin, tan, acos, asin, atan
+from ufl.operators import sqrt, exp, ln, cos, sin, tan, acos, asin, atan, \
+    erf, bessel_J, bessel_Y, bessel_I, bessel_K
 from ufl.algorithms.traversal import iter_expressions
 from ufl.algorithms.analysis import extract_type
 from ufl.algorithms.transformations import expand_compounds, Transformer, transform, transform_integrands
 
-from ufl.differentiation import is_spatially_constant
-
 class ForwardAD(Transformer):
     def __init__(self, spatial_dim, var_shape, var_free_indices, var_index_dimensions, cache=None):
         Transformer.__init__(self)
@@ -275,16 +277,23 @@ class ForwardAD(Transformer):
         # then the derivative suddenly gets accumulated,
         # which is completely wrong!
         if self._var_free_indices:
-            i0, = self._var_free_indices
-            i1, = i
-            if i == i1:
-                error("Index scope collision. Try not reusing indices for multiple expressions.\n"\
-                      "An example where this occurs is (v[i]*v[i]).dx(i)*v[i] where the index i\n"\
-                      "is bound to an index sum inside the derivative w.r.t. x[i]")
+            if i[0] in self._var_free_indices:
+                error("Index scope collision. Work around this by reusing indices less in different expressions.\n"\
+                      "An example where this occurs is (v[i]*v[i]).dx(i) where the same index i\n"\
+                      "is bound to an index sum inside the derivative .dx(i).")
+
+                # TODO: OR get around this by temporarily setting _var_free_indices to nothing?
+                #store = self._var_free_indices
+                #self._var_free_indices = nothing?
+                #visit children
+                #self._var_free_indices = store
+                # But... What would this mean? Will it be correct?
+
                 # TODO: Get around this with relabeling algorithm!
                 # j = Index()
                 # A = relabel(A, { i0: j })
                 # i = (j,)
+
         A2, Ap = self.visit(A)
         o = self.reuse_if_possible(o, A2, i)
         op = IndexSum(Ap, i)
@@ -342,6 +351,8 @@ class ForwardAD(Transformer):
     def power(self, o, a, b):
         f, fp = a
         g, gp = b
+
+        # Debugging prints, should never happen:
         if not is_true_ufl_scalar(f):
             print ":"*80
             print "f =", str(f)
@@ -350,44 +361,43 @@ class ForwardAD(Transformer):
         ufl_assert(is_true_ufl_scalar(f), "Expecting scalar expression f in f**g.")
         ufl_assert(is_true_ufl_scalar(g), "Expecting scalar expression g in f**g.")
 
-        # General case: o = f(x)**g(x)
-
+        # Derivation of the general case: o = f(x)**g(x)
+        #
         #do_df = g * f**(g-1)
         #do_dg = ln(f) * f**g
         #op = do_df*fp + do_dg*gp
-
+        #
         #do_df = o * g / f # f**g * g / f
         #do_dg = ln(f) * o
         #op = do_df*fp + do_dg*gp
 
         # Got two possible alternatives here:
-        if False:
-            # Pulling o out gives:
-            op = o*(fp*g/f + ln(f)*gp)
-            # This produces expressions like (1/w)*w**5 instead of w**4
-            # If we do this, we reuse o
-            o = self.reuse_if_possible(o, f, g)
-        else:
+        if True: # This version looks better.
             # Rewriting o as f*f**(g-1) we can do:
             f_g_m1 = f**(g-1)
             op = f_g_m1*(fp*g + f*ln(f)*gp)
             # In this case we can rewrite o using new subexpression
             o = f*f_g_m1
+        else:
+            # Pulling o out gives:
+            op = o*(fp*g/f + ln(f)*gp)
+            # This produces expressions like (1/w)*w**5 instead of w**4
+            # If we do this, we reuse o
+            o = self.reuse_if_possible(o, f, g)
 
         return (o, op)
 
     def abs(self, o, a):
         f, fprime = a
         o = self.reuse_if_possible(o, f)
-        oprime = conditional(eq(f, 0),
-                             0,
-                             Product(sign(f), fprime)) #conditional(lt(f, 0), -fprime, fprime))
+        #oprime = conditional(eq(f, 0), 0, Product(sign(f), fprime))
+        oprime = sign(f)*fprime
         return (o, oprime)
 
     # --- Mathfunctions
 
     def math_function(self, o, a):
-        if hasattr(o, 'derivative'):
+        if hasattr(o, 'derivative'): # FIXME: Introduce a UserOperator type instead of this hack
             f, fp = a
             o = self.reuse_if_possible(o, f)
             op = fp * o.derivative()
@@ -448,29 +458,74 @@ class ForwardAD(Transformer):
         op = fp/(1.0 + f**2)
         return (o, op)
 
+    def erf(self, o, a):
+        f, fp = a
+        o = self.reuse_if_possible(o, f)
+        op = fp*(2.0/sqrt(pi)*exp(-f**2))
+        return (o, op)
+
+    def bessel_j(self, o, nu, x):
+        nu, dummy = nu
+        if not (dummy is None or isinstance(dummy, Zero)):
+            error("Differentiation of bessel function w.r.t. nu is not supported.")
+        f, fp = x
+        o = self.reuse_if_possible(o, nu, f)
+        if nu == 0:
+            op = -bessel_J(1, f)
+        else:
+            op = 0.5 * (bessel_J(nu-1, f) - bessel_J(nu+1, f))
+        return (o, op*fp)
+
+    def bessel_y(self, o, nu, x):
+        nu, dummy = nu
+        if not (dummy is None or isinstance(dummy, Zero)):
+            error("Differentiation of bessel function w.r.t. nu is not supported.")
+        f, fp = x
+        o = self.reuse_if_possible(o, nu, f)
+        if nu == 0:
+            op = -bessel_Y(1, f)
+        else:
+            op = 0.5 * (bessel_Y(nu-1, f) - bessel_Y(nu+1, f))
+        return (o, op*fp)
+
+    def bessel_i(self, o, nu, x):
+        nu, dummy = nu
+        if not (dummy is None or isinstance(dummy, Zero)):
+            error("Differentiation of bessel function w.r.t. nu is not supported.")
+        f, fp = x
+        o = self.reuse_if_possible(o, nu, f)
+        if nu == 0:
+            op = bessel_I(1, f)
+        else:
+            op = 0.5 * (bessel_I(nu-1, f) + bessel_I(nu+1, f))
+        return (o, op*fp)
+
+    def bessel_k(self, o, nu, x):
+        nu, dummy = nu
+        if not (dummy is None or isinstance(dummy, Zero)):
+            error("Differentiation of bessel function w.r.t. nu is not supported.")
+        f, fp = x
+        o = self.reuse_if_possible(o, nu, f)
+        if nu == 0:
+            op = -bessel_K(1, f)
+        else:
+            op = -0.5 * (bessel_K(nu-1, f) + bessel_K(nu+1, f))
+        return (o, op*fp)
+
     # --- Restrictions
 
-    def positive_restricted(self, o, a):
-        # TODO: Assuming here that restriction and differentiation commutes. Is this correct?
+    def restricted(self, o, a):
+        # Restriction and differentiation commutes.
         f, fp = a
         o = self.reuse_if_possible(o, f)
         if isinstance(fp, ConstantValue):
-            return (o, fp)
+            return (o, fp) # TODO: Necessary? Can't restriction simplify directly instead?
         else:
-            return (o, fp('+'))
-
-    def negative_restricted(self, o, a):
-        # TODO: Assuming here that restriction and differentiation commutes. Is this correct?
-        f, fp = a
-        o = self.reuse_if_possible(o, f)
-        if isinstance(fp, ConstantValue):
-            return (o, fp)
-        else:
-            return (o, fp('-'))
+            return (o, fp(o._side)) # (f+-)' == (f')+-
 
     # --- Conditionals
 
-    def condition(self, o, l, r):
+    def binary_condition(self, o, l, r):
         o = self.reuse_if_possible(o, l[0], r[0])
         if any(not isinstance(op[1], Zero) for op in (l, r)):
             warning("Differentiating a conditional with a condition "\
@@ -479,6 +534,15 @@ class ForwardAD(Transformer):
         oprime = None # Shouldn't be used anywhere
         return (o, oprime)
 
+    def not_condition(self, o, c):
+        o = self.reuse_if_possible(o, c[0])
+        if not isinstance(c[1], Zero):
+            warning("Differentiating a conditional with a condition "\
+                        "that depends on the differentiation variable."\
+                        "This is probably not a good idea!")
+        oprime = None # Shouldn't be used anywhere
+        return (o, oprime)
+
     def conditional(self, o, c, t, f):
         o = self.reuse_if_possible(o, c[0], t[0], f[0])
         if isinstance(t[1], Zero) and isinstance(f[1], Zero):
@@ -497,12 +561,13 @@ class ForwardAD(Transformer):
     def spatial_derivative(self, o):
         # If we hit this type, it has already been propagated
         # to a terminal, so we can simply apply our derivative
-        # to its operand since differentiation commutes. Right?
+        # to its operand since differentiation commutes.
         f, ii = o.operands()
         f, fp = self.visit(f)
         o = self.reuse_if_possible(o, f, ii)
 
-        if is_spatially_constant(fp):
+        # FIXME: Make plenty of test cases around this kind of situation to document what's going on...
+        if fp.is_cellwise_constant():
             sh = fp.shape()
             fi = fp.free_indices()
             idims = fp.index_dimensions()
@@ -511,6 +576,7 @@ class ForwardAD(Transformer):
                 fi = fi + (j,)
                 idims.update(ii.index_dimensions())
             oprime = Zero(sh, fi, idims)
+            #oprime = self._make_zero_diff(o) # FIXME: Can we just use this?
         else:
             oprime = SpatialDerivative(fp, ii)
         return (o, oprime)
@@ -524,37 +590,39 @@ class SpatialAD(ForwardAD):
         else:
             vfi = ()
             vid = {}
-        ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=vfi, var_index_dimensions=vid, cache=cache)
+        ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=vfi,
+                           var_index_dimensions=vid, cache=cache)
         self._index = index
 
     def spatial_coordinate(self, o):
         # Need to define dx_i/dx_j = delta_ij?
-        I = Identity(self._spatial_dim)
-        oprime = I[:, self._index] # TODO: Is this right?
-        return (o, oprime)
-
-    def argument(self, o):
-        # Using this index in here may collide with the same index on the outside! (There's a check for this situation in index_sum above.)
-        #oprime = o.dx(self._index)
-        oprime = SpatialDerivative(o, self._index)
-        return (o, oprime)
-
-    def coefficient(self, o):
-        # Using this index in here may collide with the same index on the outside! (There's a check for this situation in index_sum above.)
-        #oprime = o.dx(self._index)
-        oprime = SpatialDerivative(o, self._index)
-        return (o, oprime)
+        if o.shape() == ():
+            return (o, IntValue(1))
+        else:
+            I = Identity(self._spatial_dim)
+            oprime = I[:, self._index]
+            return (o, oprime)
 
+    # This is implicit for all terminals, but just to make this clear:
+    facet_normal = ForwardAD.terminal # returns zero
     constant = ForwardAD.terminal # returns zero
 
-    def facet_normal(self, o):
-        #if o.cell().degree() > 1:
-        #    warning("Treating facet normal as a constant in differentiation!")
-        return ForwardAD.terminal(self, o) # returns zero
+    def form_argument(self, o):
+        # Using this index in here may collide with the
+        # same index on the outside! (There's a check for
+        # this situation in index_sum above.)
+        #oprime = o.dx(self._index) # Not using this syntax because it would create a new IndexSum
+        if o.is_cellwise_constant():
+            return self.terminal(o)
+        oprime = SpatialDerivative(o, self._index)
+        return (o, oprime)
 
 class VariableAD(ForwardAD):
     def __init__(self, spatial_dim, var, cache=None):
-        ForwardAD.__init__(self, spatial_dim, var_shape=var.shape(), var_free_indices=var.free_indices(), var_index_dimensions=var.index_dimensions(), cache=cache)
+        ForwardAD.__init__(self, spatial_dim, var_shape=var.shape(),
+                           var_free_indices=var.free_indices(),
+                           var_index_dimensions=var.index_dimensions(),
+                           cache=cache)
         self._variable = var
 
     def variable(self, o):
@@ -581,24 +649,64 @@ class VariableAD(ForwardAD):
 
 class CoefficientAD(ForwardAD):
     "Apply AFD (Automatic Functional Differentiation) to expression."
-    def __init__(self, spatial_dim, coefficients, arguments, cache=None):
-        ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=(), var_index_dimensions={}, cache=cache)
-        self._functions = zip(coefficients, arguments)
+    def __init__(self, spatial_dim, coefficients, arguments, coefficient_derivatives, cache=None):
+        ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=(),
+                           var_index_dimensions={}, cache=cache)
         self._v = arguments
         self._w = coefficients
-        ufl_assert(isinstance(self._w, Tuple), "Eh?")
-        ufl_assert(isinstance(self._v, Tuple), "Eh?")
-        # Define dw/dw := v (what we really mean by d/dw is d/dw_j where w = sum_j w_j phi_j, and what we really mean by v is phi_j for any j)
+        self._cd = coefficient_derivatives
+        ufl_assert(isinstance(self._w, Tuple), "Expecting a Tuple.")
+        ufl_assert(isinstance(self._v, Tuple), "Expecting a Tuple.")
 
     def coefficient(self, o):
+        # Define dw/dw := d/ds [w + s v] = v
+
         debug("In CoefficientAD.coefficient:")
         debug("o = %s" % o)
         debug("self._w = %s" % self._w)
         debug("self._v = %s" % self._v)
-        for (w, v) in zip(self._w, self._v): #self._functions:
+
+        # Find o among w
+        for (w, v) in zip(self._w, self._v):
             if o == w:
                 return (w, v)
-        return (o, Zero(o.shape()))
+
+        # If o is not among coefficient derivatives, return do/dw=0
+        oprimesum = Zero(o.shape())
+        oprimes = self._cd._data.get(o)
+        if oprimes is None:
+            if self._cd._data:
+                # TODO: Make it possible to silence this message in particular?
+                #       It may be good to have for debugging...
+                warning("Assuming d{%s}/d{%s} = 0." % (o, self._w))
+        else:
+            # Make sure we have a tuple to match the self._v tuple
+            if not isinstance(oprimes, tuple):
+                oprimes = (oprimes,)
+                ufl_assert(len(oprimes) == len(self._v), "Got a tuple of arguments, "+\
+                               "expecting a matching tuple of coefficient derivatives.")
+
+            # Compute do/dw_j = do/dw_h : v.
+            # Since we may actually have a tuple of oprimes and vs in a
+            # 'mixed' space, sum over them all to get the complete inner
+            # product. Using indices to define a non-compound inner product.
+            for (oprime, v) in zip(oprimes, self._v):
+                so, oi = as_scalar(oprime)
+                rv = len(v.shape())
+                oi1 = oi[:-rv]
+                oi2 = oi[-rv:]
+                prod = so*v[oi2]
+                if oi1:
+                    oprimesum += as_tensor(prod, oi1)
+                else:
+                    oprimesum += prod
+
+        # Example:
+        # (f : g) -> (dfdu : v) : g + ditto
+        # shape(f) == shape(g) == shape(dfdu : v)
+        # shape(dfdu) == shape(f) + shape(v)
+
+        return (o, oprimesum)
 
     def variable(self, o):
         # Check variable cache to reuse previously transformed variable if possible
@@ -632,8 +740,8 @@ def compute_variable_forward_ad(expr, dim):
     return ediff
 
 def compute_coefficient_forward_ad(expr, dim):
-    f, w, v = expr.operands()
-    alg = CoefficientAD(dim, w, v)
+    f, w, v, cd = expr.operands()
+    alg = CoefficientAD(dim, w, v, cd)
     e, ediff = alg.visit(f)
     return ediff
 
@@ -652,19 +760,16 @@ def forward_ad(expr, dim):
     return result
 
 
-#
 # TODO: We could expand only the compound objects that have no rule
 #       before differentiating, to allow the AD to work on a coarser graph
-#       (Missing rules for: Cross, Determinant, Cofactor)
-#
-class UnusedADRules(ForwardAD):
+class UnusedADRules(object):
 
     def _variable_derivative(self, o, f, v):
         f, fp = f
         v, vp = v
         ufl_assert(isinstance(vp, Zero), "TODO: What happens if vp != 0, i.e. v depends the differentiation variable?")
         # Are there any issues with indices here? Not sure, think through it...
-        oprime = type(o)(fp, v)
+        oprime = o.reconstruct(fp, v)
         return (o, oprime)
 
     # --- Compound operators
@@ -672,21 +777,22 @@ class UnusedADRules(ForwardAD):
     def commute(self, o, a):
         "This should work for all single argument operators that commute with d/dw."
         aprime = a[1]
-        return (o, type(o)(aprime))
+        return (o, o.reconstruct(aprime))
 
     transposed = commute
     trace = commute
     deviatoric = commute
 
+    # TODO: nabla_div, nabla_grad
     div  = commute
     curl = commute
     def grad(self, o, a):
         a, aprime = a
-        if aprime.cell() is None:
-            error("TODO: Shape of gradient is undefined.") # Currently calling expand_compounds before AD to avoid this
-            oprime = Zero(TODO)
+        c = aprime.cell()
+        if c is None or c.is_undefined():
+            oprime = self._make_zero_diff(o)
         else:
-            oprime = type(o)(aprime)
+            oprime = o.reconstruct(aprime)
         return (o, oprime)
 
     def outer(self, o, a, b):
@@ -704,6 +810,8 @@ class UnusedADRules(ForwardAD):
         b, bp = b
         return (o, dot(ap, b) + dot(a, bp))
 
+class UnimplementedADRules(object):
+
     def cross(self, o, a, b):
         error("Derivative of cross product not implemented, apply expand_compounds before AD.")
         u, up = a
