@@ -12,23 +12,25 @@ all relevant operands for use with reverse mode AD."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Kristian B. Oelgaard, 2011
 #
 # First added:  2009-01-06
-# Last changed: 2009-02-24
+# Last changed: 2011-10-21
 
+from math import pi
 from ufl.log import error
 from ufl.assertions import ufl_assert
 from ufl.classes import Zero, IntValue, FloatValue
-from ufl.operators import cos, sin, tan, acos, asin, atan, exp, ln, sqrt, conditional, sign
+from ufl.operators import cos, sin, tan, acos, asin, atan, exp, ln, sqrt, conditional, sign, erf
 from ufl.tensors import unit_vectors, ListTensor
 from ufl.algorithms.transformations import MultiFunction
 
-
 class PartialDerivativeComputer(MultiFunction):
     """NB! The main reason for keeping this out of the Expr hierarchy is
     to avoid user mistakes in the form of mixups with total derivatives,
@@ -137,9 +139,17 @@ class PartialDerivativeComputer(MultiFunction):
         "d/dx atan x = 1/(1 + x^2)"
         x, = f.operands()
         return (1.0/(1.0 + x**2),)
+
+    def erf(self, f):
+        "d/dx erf x = 2/sqrt(pi)*exp(-x^2)"
+        x, = f.operands()
+        return (2.0/sqrt(pi)*exp(-x**2),)
+
+    def bessel_function(self, nu, x):
+        return NotImplemented
     
     # --- Shape and indexing manipulators
-    
+
     def indexed(self, f): # TODO: Is this right? Fix for non-scalars too.
         "d/dx x_i = (1)_i = 1"
         s = f.shape()
