@@ -13,16 +13,16 @@ converting UFL expressions to other representations."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009-2010
 #
 # First added:  2008-05-07
-# Last changed: 2010-10-25
+# Last changed: 2011-10-24
 
 from itertools import izip
 from inspect import getargspec
@@ -37,6 +37,7 @@ from ufl.form import Form
 from ufl.integral import Integral
 from ufl.classes import all_ufl_classes
 from ufl.algorithms.analysis import has_type, extract_duplications
+from ufl.constantvalue import as_ufl
 
 def transform(expression, handlers):
     """Convert a UFLExpression according to rules defined by
@@ -62,7 +63,10 @@ class MultiFunction(object):
             cache_data = [None]*len(all_ufl_classes)
             # For all UFL classes
             for classobject in all_ufl_classes:
-                # Iterate over the inheritance chain (NB! This assumes that all UFL classes inherits a single Expr subclass and that this is the first superclass!)
+                # Iterate over the inheritance chain
+                # (NB! This assumes that all UFL classes inherits from
+                # a single Expr subclass and that the first superclass
+                # is always from the UFL Expr hierarchy!)
                 for c in classobject.mro():
                     # Register classobject with handler for the first encountered superclass
                     name = c._handlername
@@ -77,6 +81,14 @@ class MultiFunction(object):
         h = self._handlers[o._classid]
         return h(o, *args, **kwargs)
 
+    def undefined(self, o):
+        "Trigger error."
+        error("No handler defined for %s." % o._uflclass.__name__)
+
+    # Set default behaviour for any Expr
+    expr = undefined
+
+
 def is_post_handler(function):
     "Is this a handler that expects transformed children as input?"
     insp = getargspec(function)
@@ -410,9 +422,9 @@ class CompoundExpander(ReuseTransformer):
     def deviatoric(self, o, A):
         sh = self._square_matrix_shape(A)
         if sh[0] == 2:
-            return as_matrix([[-1/2*A[1,1]+1/2*A[0,0],A[0,1]],[A[1,0],1/2*A[1,1]-1/2*A[0,0]]])
+            return as_matrix([[-1./2*A[1,1]+1./2*A[0,0],A[0,1]],[A[1,0],1./2*A[1,1]-1./2*A[0,0]]])
         elif sh[0] == 3:
-            return as_matrix([[-1/3*A[1,1]-1/3*A[2,2]+2/3*A[0,0],A[0,1],A[0,2]],[A[1,0],2/3*A[1,1]-1/3*A[2,2]-1/3*A[0,0],A[1,2]],[A[2,0],A[2,1],-1/3*A[1,1]+2/3*A[2,2]-1/3*A[0,0]]])
+            return as_matrix([[-1./3*A[1,1]-1./3*A[2,2]+2./3*A[0,0],A[0,1],A[0,2]],[A[1,0],2./3*A[1,1]-1./3*A[2,2]-1./3*A[0,0],A[1,2]],[A[2,0],A[2,1],-1./3*A[1,1]+2./3*A[2,2]-1./3*A[0,0]]])
         error("dev(A) not implemented for dimension %s." % sh[0])
 
     def skew(self, o, A):
@@ -461,7 +473,6 @@ class CompoundExpander(ReuseTransformer):
         if sh[0] == 2:
             return det2D(A, 0, 1, 0, 1)
         if sh[0] == 3:
-            # TODO: Verify signs in this expression
             return A[0,0]*det2D(A, 1, 2, 1, 2) + \
                    A[0,1]*det2D(A, 1, 2, 2, 0) + \
                    A[0,2]*det2D(A, 1, 2, 0, 1)
@@ -511,7 +522,7 @@ class CompoundExpander(ReuseTransformer):
     def inverse(self, o, A):
         if A.rank() == 0:
             return 1.0 / A
-        return self.cofactor(None, A) / self.determinant(None, A)  # TODO: Verify this expression. Ainv = Acofac / detA
+        return self.cofactor(None, A) / self.determinant(None, A)
 
     # ------------ Compound differential operators
 
@@ -524,8 +535,22 @@ class CompoundExpander(ReuseTransformer):
         jj = Index()
         if a.rank() > 0:
             ii = tuple(indices(a.rank()))
-            return as_tensor(a[ii].dx(jj), tuple(ii + (jj,)))
-        return as_tensor(a.dx(jj), (jj,))
+            return as_tensor(a[ii].dx(jj), ii + (jj,))
+        else:
+            return as_tensor(a.dx(jj), (jj,))
+
+    def nabla_div(self, o, a):
+        i = Index()
+        g = a[i] if a.rank() == 1 else a[i,...]
+        return g.dx(i)
+
+    def nabla_grad(self, o, a):
+        j = Index()
+        if a.rank() > 0:
+            ii = tuple(indices(a.rank()))
+            return as_tensor(a[ii].dx(j), (j,) + ii)
+        else:
+            return as_tensor(a.dx(j), (j,))
 
     def curl(self, o, a):
         # o = curl a = "[a.dx(1), -a.dx(0)]"            if a.shape() == ()
@@ -703,39 +728,79 @@ class DuplicationPurger(ReuseTransformer):
         return e
 
 class SumDegreeEstimator(Transformer):
+    "This algorithm is exact for a few operators and heuristic for many."
 
     def __init__(self, default_degree):
         Transformer.__init__(self)
         self.default_degree = default_degree
 
     def terminal(self, v):
+        "Most terminals are spatially constant."
         return 0
 
-    def expr(self, v, *ops):
-        return max(ops)
+    def spatial_coordinate(self, v):
+        "A coordinate provides one additional degree."
+        return 1
 
     def form_argument(self, v):
-        return v.element().degree()
+        """A form argument provides a degree depending on the element,
+        or the default degree if the element has no degree."""
+        d = v.element().degree()
+        return self.default_degree if d is None else d
+
+    def expr(self, v, *ops):
+        "For most operators we take the max degree of its operands."
+        return max(ops)
 
     def spatial_derivative(self, v, f, i):
+        "A spatial derivative reduces the degree with one."
         return max(f - 1, 0)
 
     def product(self, v, *ops):
-        degrees = [op for op in ops if not op is None]
-        nones = [op for op in ops if op is None]
-        return sum(degrees) + self.default_degree*len(nones)
+        "Using the sum here is exact."
+        return sum(ops)
+
+    def division(self, v, *ops):
+        "Using the sum here is a heuristic. Consider e.g. (x+1)/(x-1)."
+        return sum(ops)
 
     def power(self, v, a, b):
+        """If b is an integer:
+        degree(a**b) == degree(a)*b
+        otherwise use the heuristic
+        degree(a**b) == degree(a)*2"""
         f, g = v.operands()
         try:
             gi = int(g)
             return a*gi
         except:
             pass
-        # Something to a non-integer power... TODO: How to handle this?
-        if b > 0:
-            return a*b
-        return a
+        # Something to a non-integer power, this is just a heuristic with no background
+        return a*2
+
+    def math_function(self, v, a):
+        """Using the heuristic
+        degree(sin(const)) == 0
+        degree(sin(a)) == degree(a)+2
+        which can be wildly inaccurate but at least
+        gives a somewhat high integration degree.
+        """
+        if a:
+            return a+2
+        else:
+            return a
+
+    def bessel_function(self, v, nu, x):
+        """Using the heuristic
+        degree(bessel_*(const)) == 0
+        degree(bessel_*(x)) == degree(x)+2
+        which can be wildly inaccurate but at least
+        gives a somewhat high integration degree.
+        """
+        if x:
+            return x+2
+        else:
+            return x
 
 class MaxDegreeEstimator(Transformer):
 
@@ -816,7 +881,8 @@ def replace(e, mapping):
     @param mapping:
         A dict with from:to replacements to perform.
     """
-    return apply_transformer(e, Replacer(mapping))
+    mapping2 = dict((k, as_ufl(v)) for (k,v) in mapping.iteritems())
+    return apply_transformer(e, Replacer(mapping2))
 
 def flatten(e): # TODO: Fix or remove! Maybe this works better now with IndexSum marking implicit summations.
     """Convert an UFL expression to a new UFL expression, with sums
@@ -858,11 +924,11 @@ def extract_argument_dependencies(e):
 
 def estimate_max_polynomial_degree(e, default_degree=1):
     """Estimate the maximum polymomial degree of all functions in the
-    expression. For coefficients defined on an element with
-    unspecified degree (None), the degree is set to the given default
-    degree."""
+    expression. For coefficients defined on an element with unspecified
+    degree (None), the degree is set to the given default degree."""
     de = MaxDegreeEstimator(default_degree)
     if isinstance(e, Form):
+        ufl_assert(e.integrals(), "Got form with no integrals!")
         degrees = [de.visit(integral.integrand()) for integral in e.integrals()]
     elif isinstance(e, Integral):
         degrees = [de.visit(e.integrand())]
@@ -876,6 +942,7 @@ def estimate_total_polynomial_degree(e, default_degree=1):
     is set to the given default degree."""
     de = SumDegreeEstimator(default_degree)
     if isinstance(e, Form):
+        ufl_assert(e.integrals(), "Got form with no integrals!")
         degrees = [de.visit(integral.integrand()) for integral in e.integrals()]
     elif isinstance(e, Integral):
         degrees = [de.visit(e.integrand())]
