@@ -13,16 +13,17 @@ converting UFL expressions to other representations."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
+# Modified by Kristian B. Oelgaard, 2011
 #
 # First added:  2008-05-07
-# Last changed: 2009-06-11
+# Last changed: 2011-10-21
 
 import os
 from itertools import chain
@@ -39,9 +40,9 @@ from ufl.tensors import ListTensor, ComponentTensor
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.indexsum import IndexSum
 from ufl.tensoralgebra import Transposed, Outer, Inner, Dot, Cross, Trace, Determinant, Inverse, Deviatoric, Cofactor
-from ufl.mathfunctions import Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan
+from ufl.mathfunctions import Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
 from ufl.restriction import PositiveRestricted, NegativeRestricted
-from ufl.differentiation import SpatialDerivative, VariableDerivative, Grad, Div, Curl
+from ufl.differentiation import SpatialDerivative, VariableDerivative, Grad, Div, Curl, NablaGrad, NablaDiv
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 from ufl.form import Form
 from ufl.integral import Measure
@@ -75,11 +76,11 @@ def build_precedence_map():
     precedence_list.append((Conditional,))
     precedence_list.append((LE, GT, GE, NE, EQ, LT))
 
-    precedence_list.append((Div, Grad, Curl, SpatialDerivative, VariableDerivative,
+    precedence_list.append((Div, Grad, NablaGrad, NablaDiv, Curl, SpatialDerivative, VariableDerivative,
                             Determinant, Trace, Cofactor, Inverse, Deviatoric))
     precedence_list.append((Product, Division, Cross, Dot, Outer, Inner))
     precedence_list.append((Indexed, Transposed, Power))
-    precedence_list.append((Abs, Cos, Exp, Ln, Sin, Sqrt, Tan, Acos, Asin, Atan))
+    precedence_list.append((Abs, Cos, Exp, Ln, Sin, Sqrt, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK))
     precedence_list.append((Variable,))
     precedence_list.append(terminal_classes)
 
@@ -203,9 +204,15 @@ class Expression2LatexHandler(Transformer):
         return r"\frac{%s}{%s}[%s]" % (nom, denom, v) # TODO: Fix this syntax...
 
     def grad(self, o, f):
-        return r"\nabla{%s}" % par(f)
+        return r"\mathbf{grad}{%s}" % par(f)
 
     def div(self, o, f):
+        return r"\mathbf{grad}{%s}" % par(f)
+
+    def nabla_grad(self, o, f):
+        return r"\nabla{\otimes %s}" % par(f)
+
+    def nabla_div(self, o, f):
         return r"\nabla{\cdot %s}" % par(f)
 
     def curl(self, o, f):
@@ -238,6 +245,21 @@ class Expression2LatexHandler(Transformer):
     def atan(self, o, f):
         return r"\arctan{%s}" % par(f)
 
+    def erf(self, o, f):
+        return r"\erf{%s}" % par(f)
+
+    def bessel_j(self, o, nu, f):
+        return r"J_{%s}{%s}" % (nu, par(f))
+
+    def bessel_y(self, o, nu, f):
+        return r"Y_{%s}{%s}" % (nu, par(f))
+
+    def bessel_i(self, o, nu, f):
+        return r"I_{%s}{%s}" % (nu, par(f))
+
+    def bessel_K(self, o, nu, f):
+        return r"K_{%s}{%s}" % (nu, par(f))
+
     def power(self, o, a, b):
         return "{%s}^{%s}" % (par(a), par(b))
 
@@ -317,6 +339,15 @@ class Expression2LatexHandler(Transformer):
     def gt(self, o, a, b):
         return "(%s > %s)" % (a, b)
 
+    def and_condition(self, o, a, b):
+        return "(%s && %s)" % (a, b)
+
+    def or_condition(self, o, a, b):
+        return "(%s || %s)" % (a, b)
+
+    def not_condition(self, o, a):
+        return "!(%s)" % (a,)
+
     def conditional(self, o, c, t, f):
         l = "\\begin{cases}\n"
         l += "%s, &\text{if }\quad %s, \\\\\n" % (t, c)
@@ -341,12 +372,14 @@ domain_strings = { Measure.CELL: r"\Omega",
                    Measure.EXTERIOR_FACET: r"\Gamma^{ext}",
                    Measure.INTERIOR_FACET: r"\Gamma^{int}",
                    Measure.MACRO_CELL: r"\Omega^{macro}",
+                   Measure.SURFACE: r"\Gamma^{surface}",
                  }
 default_domain_string = "D"
 dx_strings = { Measure.CELL: "dx",
                Measure.EXTERIOR_FACET: "ds",
                Measure.INTERIOR_FACET: "dS",
                Measure.MACRO_CELL: "dE",
+               Measure.SURFACE: "dc",
              }
 
 def form2latex(form, formdata):
@@ -392,8 +425,10 @@ def form2latex(form, formdata):
     integrals = list(chain(form.cell_integrals(),
                            form.exterior_facet_integrals(),
                            form.interior_facet_integrals(),
-                           form.macro_cell_integrals()))
-    ufl_assert(len(integrals) == len(form.integrals()), "FIXME: Not handling non-standard integral types!")
+                           form.macro_cell_integrals(),
+                           form.surface_integrals()))
+    ufl_assert(len(integrals) == len(form.integrals()),
+               "Not handling non-standard integral types!")
     lines = []
     for itg in integrals:
         variables = extract_variables(itg.integrand())
