@@ -12,16 +12,16 @@ classes (functions), including TestFunction and TrialFunction."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
 #
 # First added:  2008-03-14
-# Last changed: 2011-05-12
+# Last changed: 2011-10-20
 
 from ufl.assertions import ufl_assert
 from ufl.common import Counted, product
@@ -32,6 +32,7 @@ from ufl.finiteelement import FiniteElementBase
 # --- Class representing an argument (basis function) in a form ---
 
 class Argument(FormArgument, Counted):
+    """UFL value: Representation of an argument to a form."""
     __slots__ = ("_repr", "_element",)
     _globalcount = 0
 
@@ -55,7 +56,7 @@ class Argument(FormArgument, Counted):
         ufl_assert(isinstance(count, int),
                    "Expecting an int, not %s" % count)
         ufl_assert(element.value_shape() == self._element.value_shape(),
-                   "Cannot reconstruct a VectorConstant with a different value shape.")
+                   "Cannot reconstruct an Argument with a different value shape.")
         return Argument(element, count)
 
     def element(self):
@@ -67,6 +68,13 @@ class Argument(FormArgument, Counted):
     def cell(self):
         return self._element.cell()
 
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        # TODO: Should in principle do like with Coefficient,
+        # but that may currently simplify away some arguments
+        # we want to keep, or?
+        return False
+
     def __str__(self):
         count = str(self._count)
         if len(count) == 1:
@@ -83,18 +91,26 @@ class Argument(FormArgument, Counted):
 # --- Helper functions for pretty syntax ---
 
 def TestFunction(element):
+    """UFL value: Create a test function argument to a form."""
     return Argument(element, -2)
 
 def TrialFunction(element):
+    """UFL value: Create a trial function argument to a form."""
     return Argument(element, -1)
 
 # --- Helper functions for creating subfunctions on mixed elements ---
 
 def Arguments(element):
+    """UFL value: Create an Argument in a mixed space, and return a
+    tuple with the function components corresponding to the subelements."""
     return split(Argument(element))
 
 def TestFunctions(element):
+    """UFL value: Create a TestFunction in a mixed space, and return a
+    tuple with the function components corresponding to the subelements."""
     return split(TestFunction(element))
 
 def TrialFunctions(element):
+    """UFL value: Create a TrialFunction in a mixed space, and return a
+    tuple with the function components corresponding to the subelements."""
     return split(TrialFunction(element))
