@@ -12,16 +12,16 @@ of related classes, including Constant."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
 #
 # First added:  2008-03-14
-# Last changed: 2011-05-11
+# Last changed: 2011-10-20
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
@@ -34,23 +34,19 @@ from ufl.geometry import as_cell
 # --- The Coefficient class represents a coefficient in a form ---
 
 class Coefficient(FormArgument, Counted):
+    """UFL form argument type: Representation of a form coefficient."""
+
     # Slots are disabled here because they cause trouble in PyDOLFIN multiple inheritance pattern:
     #__slots__ = ("_element", "_repr", "_gradient", "_derivatives")
     _globalcount = 0
 
-    def __init__(self, element, count=None):#, gradient=None, derivatives=None):
+    def __init__(self, element, count=None):
         FormArgument.__init__(self)
         Counted.__init__(self, count, Coefficient)
         ufl_assert(isinstance(element, FiniteElementBase),
             "Expecting a FiniteElementBase instance.")
         self._element = element
         self._repr = None
-        #self._gradient = gradient
-        #self._derivatives = {} if derivatives is None else dict(derivatives)
-        #if gradient or derivatives:
-        #    # TODO: Use gradient and derivatives in AD
-        #    # TODO: Check shapes of gradient and derivatives
-        #    warning("Specifying the gradient or derivatives of a Coefficient is not yet implemented anywhere.")
 
     def reconstruct(self, element=None, count=None):
         # This code is shared with the FooConstant classes
@@ -65,27 +61,23 @@ class Coefficient(FormArgument, Counted):
         ufl_assert(isinstance(count, int),
                    "Expecting an int, not %s" % count)
         ufl_assert(element.value_shape() == self._element.value_shape(),
-                   "Cannot reconstruct a VectorConstant with a different value shape.")
+                   "Cannot reconstruct a Coefficient with a different value shape.")
         return self._reconstruct(element, count)
 
     def _reconstruct(self, element, count):
         # This code is class specific
         return Coefficient(element, count)
 
-    #def gradient(self):
-    #    "Hook for experimental feature, do not use!"
-    #    return self._gradient
-
-    #def derivative(self, f):
-    #    "Hook for experimental feature, do not use!"
-    #    return self._derivatives.get(f)
-
     def element(self):
         return self._element
 
     def shape(self):
         return self._element.value_shape()
 
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return self._element.is_cellwise_constant()
+
     def cell(self):
         return self._element.cell()
 
@@ -106,18 +98,17 @@ class Coefficient(FormArgument, Counted):
 
 # --- Subclasses for defining constant coefficients without specifying element ---
 
-# TODO: Handle actual global constants?
-
 class ConstantBase(Coefficient):
     __slots__ = ()
     def __init__(self, element, count):
         Coefficient.__init__(self, element, count)
 
 class Constant(ConstantBase):
+    """UFL value: Represents a globally constant scalar valued coefficient."""
     __slots__ = ()
 
     def __init__(self, cell, count=None):
-        e = FiniteElement("DG", cell, 0)
+        e = FiniteElement("Real", cell, 0)
         ConstantBase.__init__(self, e, count)
         self._repr = "Constant(%r, %r)" % (e.cell(), self._count)
 
@@ -132,11 +123,13 @@ class Constant(ConstantBase):
             return "c_{%s}" % count
 
 class VectorConstant(ConstantBase):
+    """UFL value: Represents a globally constant vector valued coefficient."""
     __slots__ = ()
 
     def __init__(self, cell, dim=None, count=None):
-        e = VectorElement("DG", cell, 0, dim)
+        e = VectorElement("Real", cell, 0, dim)
         ConstantBase.__init__(self, e, count)
+        ufl_assert(self._repr is None, "Repr should not have been set yet!")
         self._repr = "VectorConstant(%r, %r, %r)" % (e.cell(), e.value_shape()[0], self._count)
 
     def _reconstruct(self, element, count):
@@ -150,10 +143,13 @@ class VectorConstant(ConstantBase):
             return "C_{%s}" % count
 
 class TensorConstant(ConstantBase):
+    """UFL value: Represents a globally constant tensor valued coefficient."""
     __slots__ = ()
+
     def __init__(self, cell, shape=None, symmetry=None, count=None):
-        e = TensorElement("DG", cell, 0, shape=shape, symmetry=symmetry)
+        e = TensorElement("Real", cell, 0, shape=shape, symmetry=symmetry)
         ConstantBase.__init__(self, e, count)
+        ufl_assert(self._repr is None, "Repr should not have been set yet!")
         self._repr = "TensorConstant(%r, %r, %r, %r)" % (e.cell(), e.value_shape(), e._symmetry, self._count)
 
     def _reconstruct(self, element, count):
@@ -170,4 +166,6 @@ class TensorConstant(ConstantBase):
 # --- Helper functions for subfunctions on mixed elements ---
 
 def Coefficients(element):
+    """UFL value: Create a Coefficient in a mixed space, and return a
+    tuple with the function components corresponding to the subelements."""
     return split(Coefficient(element))
