@@ -11,16 +11,16 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian Oelgaard, 2009
 #
 # First added:  2008-08-05
-# Last changed: 2010-02-12
+# Last changed: 2011-06-02
 
 import os
 from itertools import izip
@@ -105,6 +105,9 @@ def split_dict(d, criteria):
             b[k] = v
     return a, b
 
+def slice_dict(dictionary, keys, default=None):
+    return tuple(dictionary.get(k, default) for k in keys)
+
 def some_key(a_dict):
     "Return an arbitrary key from a dictionary."
     return zip((0,), a_dict.iterkeys())[0][1]
