@@ -11,7 +11,7 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
@@ -20,7 +20,7 @@
 # Modified by Anders Logg, 2011.
 #
 # First added:  2008-11-01
-# Last changed: 2009-09-22
+# Last changed: 2011-11-10
 
 from ufl.log import warning, error
 from ufl.assertions import ufl_assert, expecting_python_scalar
@@ -42,18 +42,15 @@ python_scalar_types = (int, float)
 #except:
 #    pass
 
-# FIXME: Consider coordinating this with set_float_formatting
-# FIXME: in FFC (move to UFL and use from FFC)
-
 # Precision for float formatting
 precision = None
 
 def format_float(x):
-    "Format float value based on global UFL precision"
+    "Format float value based on global UFL precision."
     if precision is None:
         return repr(x)
     else:
-        return ("%%.%df" % precision) % x
+        return ("%%.%dg" % precision) % x
 
 #--- Base classes for constant types ---
 
@@ -61,6 +58,10 @@ class ConstantValue(Terminal):
     def __init__(self):
         Terminal.__init__(self)
 
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return True
+
 class IndexAnnotated(object):
     """Class to annotate expressions that don't depend on
     indices with a set of free indices, used internally to keep
@@ -87,6 +88,7 @@ class IndexAnnotated(object):
 #--- Class for representing zero tensors of different shapes ---
 
 class Zero(ConstantValue, IndexAnnotated):
+    "UFL literal type: Representation of a zero valued expression."
     __slots__ = ()
 
     def __init__(self, shape=(), free_indices=(), index_dimensions=None):
@@ -145,7 +147,7 @@ class Zero(ConstantValue, IndexAnnotated):
         return 0
 
 def zero(*shape):
-    "Return a zero tensor with the given shape."
+    "UFL literal constant: Return a zero tensor with the given shape."
     return Zero(shape)
 
 #--- Scalar value types ---
@@ -208,7 +210,7 @@ class ScalarValue(ConstantValue, IndexAnnotated):
         return type(self)(abs(self._value))
 
 class FloatValue(ScalarValue):
-    "A constant scalar numeric value."
+    "UFL literal type: Representation of a constant scalar floating point value."
     __slots__ = ()
     def __init__(self, value, shape=(), free_indices=(), index_dimensions=None):
 
@@ -228,11 +230,13 @@ class FloatValue(ScalarValue):
         return self._repr
 
 class IntValue(ScalarValue):
-    "A constant scalar integer value."
+    "UFL literal type: Representation of a constant scalar integer value."
     __slots__ = ()
     def __init__(self, value, shape=(), free_indices=(), index_dimensions=None):
         ScalarValue.__init__(self, int_type(value), shape, free_indices, index_dimensions)
-        self._repr = "%s(%s, %s, %s, %s)" % (type(self).__name__, repr(self._value), repr(self._shape), repr(self._free_indices), repr(self._index_dimensions))
+        self._repr = "%s(%s, %s, %s, %s)" % (type(self).__name__, repr(self._value),
+                                             repr(self._shape), repr(self._free_indices),
+                                             repr(self._index_dimensions))
 
     def __repr__(self):
         return self._repr
@@ -240,6 +244,7 @@ class IntValue(ScalarValue):
 #--- Identity matrix ---
 
 class Identity(ConstantValue):
+    "UFL literal type: Representation of an identity matrix."
     __slots__ = ("_dim", "_repr")
 
     def __new__(cls, dim):
@@ -278,6 +283,10 @@ class Identity(ConstantValue):
 #--- Permutation symbol ---
 
 class PermutationSymbol(ConstantValue):
+    """UFL literal type: Representation of a permutation symbol.
+
+    This is also known as the Levi-Civita symbol, antisymmetric symbol,
+    or alternating symbol."""
     __slots__ = ("_dim", "_repr")
 
     def __init__(self, dim):
@@ -310,7 +319,8 @@ class PermutationSymbol(ConstantValue):
         return isinstance(other, PermutationSymbol) and self._dim == other._dim
 
     def __eps(self, x):
-        """ This function body is taken from http://www.mathkb.com/Uwe/Forum.aspx/math/29865/N-integer-Levi-Civita"""
+        """This function body is taken from
+        http://www.mathkb.com/Uwe/Forum.aspx/math/29865/N-integer-Levi-Civita"""
         result = IntValue(1)
         for i, x1 in enumerate(x):
            for j in xrange(i + 1, len(x)):
@@ -346,4 +356,5 @@ def as_ufl(expression):
         return IntValue(expression)
     if isinstance(expression, (float, float_type)):
         return FloatValue(expression)
-    error("Invalid type conversion: %s can not be converted to any UFL type.\nThe representation of the object is:\n%r" % (type(expression), expression))
+    error(("Invalid type conversion: %s can not be converted to any UFL type.\n"+\
+           "The representation of the object is:\n%r") % (type(expression), expression))
