@@ -19,16 +19,16 @@ This is to avoid circular dependencies between Expr and its subclasses.
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008
 #
 # First added:  2008-03-14
-# Last changed: 2011-04-12
+# Last changed: 2011-08-10
 
 #--- The base object for all UFL expression tree nodes ---
 
@@ -48,13 +48,22 @@ class Expr(object):
     "Base class for all UFL objects."
     # Freeze member variables for objects of this class
     __slots__ = ()
-    #__slots__ = ("_operands", "_hash", "_str", "_repr", "_shape", "_free_indices", "_index_dimensions")
     
     def __init__(self):
         # Comment out this line to disable class construction statistics (used in some unit tests)
         _class_usage_statistics[self.__class__._uflclass] += 1
         #self._hash = None
-    
+
+    #def _lock(self): # TODO: Try this out on most types
+    #    "Call this after constructing object to make it forcibly immutable."
+    #    self._locked = True
+
+    #def __setattr__(self, name, value):
+    #    if hasattr(self, '_locked') and self._locked:
+    #        raise Exception("%s is immutable, cannot set property %s." % (type(self), name))
+    #    else:
+    #        return object.__init__(self, name, value)
+
     #=== Abstract functions that must be implemented by subclasses ===
     
     #--- Functions for reconstructing expression ---
@@ -98,6 +107,20 @@ class Expr(object):
                     break
         return c
 
+    def geometric_dimension(self):
+        "Return the geometric dimension this expression lives in."
+        # This function was introduced to clarify and
+        # eventually reduce direct dependencies on cells.
+        cell = self.cell()
+        if cell is None or cell.is_undefined():
+            error("Cannot infer geometric dimension for this expression.")
+        else:
+            return cell.geometric_dimension()
+
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        raise NotImplementedError(self.__class__.is_cellwise_constant)
+
     #--- Functions for float evaluation ---
     
     def evaluate(self, x, mapping, component, index_values):
@@ -151,19 +174,21 @@ class Expr(object):
         """Checks whether the two expressions are represented the
         exact same way using repr. This does not check if the forms
         are mathematically equal or equivalent! Used by sets and dicts."""
-        #return (type(self) == type(other)) and ((self is other) or (self.operands() == other.operands()))
+        #return (type(self) == type(other)) and ((self is other) \
+        #      or (self.operands() == other.operands()))
         return repr(self) == repr(other)
 
     def __nonzero__(self):
         "By default, all Expr are nonzero."
         return True 
-    
+
     def __len__(self):
         "Length of expression. Used for iteration over vector expressions."
         s = self.shape()
         if len(s) == 1:
             return s[0]
-        error("Cannot take length of non-vector expression.")
+        #error("Cannot take length of non-vector expression.")
+        raise NotImplementedError("Cannot take length of non-vector expression.")
     
     def __iter__(self):
         "Iteration over vector expressions."
@@ -188,6 +213,10 @@ class Operator(Expr):
         "Return a new object of the same type with new operands."
         return self.__class__._uflclass(*operands)
 
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return all(o.is_cellwise_constant() for o in self.operands())
+
 class WrapperType(Operator):
     def __init__(self):
         Operator.__init__(self)
