@@ -13,14 +13,14 @@ Sum and its superclass Expr."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2008-08-18
-# Last changed: 2011-04-12
+# Last changed: 2011-10-25
 
 from itertools import chain
 
@@ -168,6 +168,7 @@ def _rdiv(self, o):
         return NotImplemented
     return Division(o, self)
 Expr.__rdiv__ = _rdiv
+Expr.__rtruediv__ = _rdiv
 
 def _pow(self, o):
     if not isinstance(o, _valid_types):
@@ -213,7 +214,12 @@ def _call(self, arg, mapping=None):
     component = ()
     index_values = StackDict()
     from ufl.algorithms import expand_derivatives
-    dim = len(arg) if isinstance(arg, (tuple, list)) else 1
+    if arg is None:
+        dim = None
+    elif isinstance(arg, (tuple, list)):
+        dim = len(arg)
+    else: # No type checking here...
+        dim = 1
     f = expand_derivatives(self, dim)
     return f.evaluate(arg, mapping, component, index_values)
 Expr.__call__ = _call
