@@ -11,17 +11,17 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009.
 # Modified by Kristian B. Oelgaard, 2009
 #
 # First added:  2008-03-14
-# Last changed: 2011-05-08
+# Last changed: 2011-11-17
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
@@ -52,7 +52,10 @@ domain2facet = {"cell1D": "vertex",
                 "hexahedron": "quadrilateral"}
 
 # Number of facets associated with each domain
-domain2num_facets = {"interval": 2,
+domain2num_facets = {"cell1D": None,
+                     "cell2D": None,
+                     "cell3D": None,
+                     "interval": 2,
                      "triangle": 3,
                      "tetrahedron": 4,
                      "quadrilateral": 4,
@@ -70,6 +73,10 @@ class GeometricQuantity(Terminal):
     def cell(self):
         return self._cell
 
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return True # NB! Assuming all geometric quantities in here are are cellwise constant by default!
+
 class SpatialCoordinate(GeometricQuantity):
     "Representation of a spatial coordinate."
     __slots__ = ("_repr",)
@@ -77,6 +84,10 @@ class SpatialCoordinate(GeometricQuantity):
         GeometricQuantity.__init__(self, cell)
         self._repr = "SpatialCoordinate(%r)" % self._cell
 
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return False
+
     def shape(self):
         d = self._cell.geometric_dimension()
         if d == 1:
@@ -84,7 +95,13 @@ class SpatialCoordinate(GeometricQuantity):
         return (d,)
 
     def evaluate(self, x, mapping, component, index_values):
-        return float(x[component[0]])
+        if component == ():
+            if isinstance(x, (tuple,list)):
+                return float(x[0])
+            else:
+                return float(x)
+        else:
+            return float(x[component[0]])
 
     def __str__(self):
         return "x"
@@ -155,6 +172,44 @@ class Circumradius(GeometricQuantity):
     def __eq__(self, other):
         return isinstance(other, Circumradius) and other._cell == self._cell
 
+class CellSurfaceArea(GeometricQuantity):
+    "Representation of the total surface area of a cell."
+    __slots__ = ("_repr",)
+    def __init__(self, cell):
+        GeometricQuantity.__init__(self, cell)
+        self._repr = "CellSurfaceArea(%r)" % self._cell
+
+    def shape(self):
+        return ()
+
+    def __str__(self):
+        return "surfacearea"
+
+    def __repr__(self):
+        return self._repr
+
+    def __eq__(self, other):
+        return isinstance(other, CellSurfaceArea) and other._cell == self._cell
+
+class FacetArea(GeometricQuantity):
+    "Representation of the area of a cell facet."
+    __slots__ = ("_repr",)
+    def __init__(self, cell):
+        GeometricQuantity.__init__(self, cell)
+        self._repr = "FacetArea(%r)" % self._cell
+
+    def shape(self):
+        return ()
+
+    def __str__(self):
+        return "facetarea"
+
+    def __repr__(self):
+        return self._repr
+
+    def __eq__(self, other):
+        return isinstance(other, FacetArea) and other._cell == self._cell
+
 # TODO: If we include this here, we must define exactly what is meant by the mesh size, possibly adding multiple kinds of mesh sizes (hmin, hmax, havg, ?)
 #class MeshSize(GeometricQuantity):
 #    def __init__(self, cell):
@@ -179,7 +234,7 @@ class Space(object):
     __slots__ = ("_dimension",)
 
     def __init__(self, dimension):
-        #ufl_assert(isinstance(dimension, int), "Expecting integer.") # FIXME: This is essential!
+        ufl_assert(isinstance(dimension, int), "Expecting integer.")
         self._dimension = dimension
 
     def dimension(self):
@@ -194,12 +249,14 @@ class Space(object):
 
 class Cell(object):
     "Representation of a finite element cell."
-    __slots__ = ("_domain", "_space", "_geometric_dimension",
-                 "_topological_dimension", "_repr", "_invalid",
-                 "d", "n", "x", "volume", "circumradius")
+    __slots__ = ("_domain", "_space",
+                 "_geometric_dimension", "_topological_dimension",
+                 "_repr", "_invalid",
+                 "_n", "_x", "_volume", "_circumradius",
+                 "_cellsurfacearea", "_facetarea",)
 
     def __init__(self, domain, space=None):
-        "Initialize basic cell description"
+        "Initialize basic cell description."
 
         # Check for valid domain. We allow None to support PyDOLFIN
         # integration features. This is a bit dangerous because
@@ -222,7 +279,8 @@ class Cell(object):
             dim = domain2dim[self._domain]
             self._topological_dimension = dim
 
-            # The space dimension defaults to equal the topological dimension if undefined
+            # The space dimension defaults to equal
+            # the topological dimension if undefined
             space = Space(dim) if space is None else space
             ufl_assert(isinstance(space, Space),
                        "Expecting a Space instance, not '%r'" % (space,))
@@ -239,48 +297,90 @@ class Cell(object):
                        "Cannot embed a %sD cell in %s" %\
                            (istr(self._topological_dimension), self._space))
 
-            # Attach a cell dimension for use in code.
-            # TODO: Make self.d a property, deprecate or make valid
-            #       only in this case. Don't use inside UFL!
-            if self._topological_dimension == self._geometric_dimension:
-                self.d = self._geometric_dimension
-            else:
-                self.d = None # TODO: Make this a property so user code can fail instead of silently getting None
-
         # Cache repr string
         self._repr = "Cell(%r, %r)" % (self._domain, self._space)
 
         # Attach expression nodes derived from this cell
-        self.n = FacetNormal(self)
-        self.x = SpatialCoordinate(self)
-        self.volume = CellVolume(self)
-        self.circumradius = Circumradius(self)
-        #self.h = MeshSize(self)
-        #self.hmin = MeshSizeMin(self)
-        #self.hmax = MeshSizeMax(self)
+        self._n = FacetNormal(self)
+        self._x = SpatialCoordinate(self)
+        self._volume = CellVolume(self)
+        self._circumradius = Circumradius(self)
+        self._cellsurfacearea = CellSurfaceArea(self)
+        self._facetarea = FacetArea(self)
+        #self._h = MeshSize(self)
+        #self._hmin = MeshSizeMin(self)
+        #self._hmax = MeshSizeMax(self)
 
-    def geometric_dimension(self):
-        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
-        return self.d
+    @property
+    def x(self):
+        "UFL geometry value: The global spatial coordinates."
+        return self._x
 
-    def topological_dimension(self):
-        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
-        return self.d
+    @property
+    def n(self):
+        "UFL geometry value: The facet normal on the cell boundary."
+        return self._n
+
+    @property
+    def volume(self):
+        "UFL geometry value: The volume of the cell."
+        return self._volume
+
+    @property
+    def circumradius(self):
+        "UFL geometry value: The circumradius of the cell."
+        return self._circumradius
+
+    @property
+    def facet_area(self):
+        "UFL geometry value: The area of a facet of the cell."
+        return self._facetarea
+
+    @property
+    def surface_area(self):
+        "UFL geometry value: The total surface area of the cell."
+        return self._cellsurfacearea
 
     def is_undefined(self):
+        """Return whether this cell is undefined,
+        in which case no dimensions are available."""
         return self._invalid
 
     def domain(self):
+        "Return the domain of the cell."
         ufl_assert(not self._invalid, "An invalid cell has no domain.")
         return self._domain
 
+    def facet_domain(self):
+        "Return the domain of the facet of this cell."
+        ufl_assert(not self._invalid, "An invalid cell has no facet domains.")
+        return domain2facet[self._domain]
+
     def num_facets(self):
+        "Return the number of facets this cell has."
         ufl_assert(not self._invalid, "An invalid cell has no facets.")
         return domain2num_facets[self._domain]
 
-    def facet_domain(self):
-        ufl_assert(not self._invalid, "An invalid cell has no facet domains.")
-        return domain2facet[self._domain]
+    def geometric_dimension(self):
+        "Return the dimension of the space this cell is embedded in."
+        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
+        return self._geometric_dimension
+
+    def topological_dimension(self):
+        "Return the dimension of the topology of this cell."
+        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
+        return self._topological_dimension
+
+    @property
+    def d(self):
+        """The dimension of the cell.
+
+        Only valid if the geometric and topological dimensions are the same."""
+        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
+        ufl_assert(self._topological_dimension == self._geometric_dimension,
+                   "Cell.d is undefined when geometric and"+\
+                   "topological dimensions are not the same.")
+        return self._geometric_dimension
 
     def __eq__(self, other):
         return isinstance(other, Cell) and repr(self) == repr(other)
@@ -300,5 +400,6 @@ class Cell(object):
 # --- Utility conversion functions
 
 def as_cell(cell):
-    "Convert any valid object to a Cell (in particular, domain string)."
+    """Convert any valid object to a Cell (in particular, domain string),
+    or return cell if it is already a Cell."""
     return cell if isinstance(cell, Cell) else Cell(cell)
