@@ -11,19 +11,19 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-01-28
-# Last changed: 2009-04-19
+# Last changed: 2011-06-17
 
 from collections import defaultdict
 from ufl.log import error
 from ufl.expr import Expr, WrapperType
-from ufl.indexing import IndexBase, Index, as_multi_index
+from ufl.indexing import IndexBase, Index, FixedIndex, as_multi_index
 from ufl.indexutils import unique_indices
 from ufl.precedence import parstr
 
@@ -37,14 +37,19 @@ class Indexed(WrapperType):
         if not isinstance(expression, Expr):
             error("Expecting Expr instance, not %s." % repr(expression))
         self._expression = expression
+
         shape = expression.shape()
-        self._indices = as_multi_index(indices, shape)
-        
-        if expression.rank() != len(self._indices):
+        if len(shape) != len(indices):
             error("Invalid number of indices (%d) for tensor "\
                 "expression of rank %d:\n\t%r\n"\
-                % (len(self._indices), expression.rank(), expression))
-        
+                % (len(indices), expression.rank(), expression))
+
+        self._indices = as_multi_index(indices, shape)
+
+        for si, di in zip(shape, self._indices):
+            if isinstance(di, FixedIndex) and int(di) >= int(si):
+                error("Fixed index out of range!")
+
         idims = dict((i, s) for (i, s) in zip(self._indices._indices, shape) if isinstance(i, Index))
         idims.update(expression.index_dimensions())
         fi = unique_indices(expression.free_indices() + self._indices._indices)
@@ -66,6 +71,10 @@ class Indexed(WrapperType):
     def shape(self):
         return ()
     
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return self._expression.is_cellwise_constant()
+
     def evaluate(self, x, mapping, component, index_values, derivatives=()):
         A, ii = self.operands()
         component = ii.evaluate(x, mapping, None, index_values)
