@@ -11,16 +11,17 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2008-03-14
-# Last changed: 2009-04-25
+# Last changed: 2011-06-22
 
 from ufl.log import error, warning
+from ufl.assertions import ufl_assert
 from ufl.common import Counted
 from ufl.terminal import UtilityType
 
@@ -31,6 +32,9 @@ class IndexBase(object):
         pass
 
 class Index(IndexBase, Counted):
+    """UFL value: An index with no value assigned.
+
+    Used to represent free indices in Einstein indexing notation."""
     __slots__ = ("_str", "_repr", "_hash")
     _globalcount = 0
     def __init__(self, count=None):
@@ -57,6 +61,7 @@ class Index(IndexBase, Counted):
         return self._repr
 
 class FixedIndex(IndexBase):
+    """UFL value: An index with a specific value assigned."""
     __slots__ = ("_value", "_repr")
     
     def __init__(self, value):
@@ -86,6 +91,7 @@ class FixedIndex(IndexBase):
         return self._repr
 
 class MultiIndex(UtilityType):
+    "Represents a sequence of indices, either fixed or free."
     __slots__ = ("_indices", "_str", "_repr")
     
     def __init__(self, ii, idims=None):
@@ -99,7 +105,7 @@ class MultiIndex(UtilityType):
             ii = tuple(as_index(j) for j in ii)
         else:
             error("Expecting tuple of UFL indices.")
-        
+
         # TODO: Remove "idims is None" when it can no longer occur
         if idims is None:
             warning("No index dimensions provided in MultiIndex.")
@@ -110,7 +116,9 @@ class MultiIndex(UtilityType):
                 if isinstance(k, Index):
                     if not k in idims:
                         error("Missing index in the provided idims.")
-
+                    else:
+                        ufl_assert(isinstance(idims[k], int),
+                                   "Non-integer index dimension provided.")
         self._indices = ii
         self._idims = idims
         self._str = ", ".join(str(i) for i in self._indices)
@@ -180,19 +188,24 @@ def as_index(i):
         return (i,)
     error("Invalid object %s to create index from." % repr(i))
 
+def _make_idims(ii, shape):
+    if shape is None:
+        return None
+    else:
+        return dict((j,d) for (j,d) in zip(ii, shape) if isinstance(j, Index))
+
 def as_multi_index(ii, shape=None):
     if isinstance(ii, MultiIndex):
-        if not ii._idims:
-            idims = None if shape is None else dict((j,d) for (j,d) in zip(ii, shape))
-            ii._idims = idims
-        return ii
-    if not isinstance(ii, tuple):
+        if ii._idims:
+            return ii
+        else:
+            ii = ii._indices
+    elif not isinstance(ii, tuple):
         ii = (ii,)
-    idims = None if shape is None else dict((j,d) for (j,d) in zip(ii, shape))
-    return MultiIndex(ii, idims)
+    return MultiIndex(ii, _make_idims(ii, shape))
 
 def indices(n):
-    "Return a tuple of n new Index objects."
+    "UFL value: Return a tuple of n new Index objects."
     return tuple(Index() for _i in range(n))
 
 # TODO: Fix imports everywhere else instead
