@@ -11,16 +11,17 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-01-28
-# Last changed: 2009-04-19
+# Last changed: 2011-06-17
 
 from ufl.log import error
+from ufl.assertions import ufl_assert
 from ufl.expr import Expr, AlgebraOperator
 from ufl.indexing import Index, MultiIndex, as_multi_index
 from ufl.precedence import parstr
@@ -31,10 +32,20 @@ class IndexSum(AlgebraOperator):
     __slots__ = ("_summand", "_index", "_dimension", "_repr", "_free_indices", "_index_dimensions")
     
     def __new__(cls, summand, index):
+        if not isinstance(summand, Expr):
+            error("Expecting Expr instance, not %s." % repr(summand))
+
         from ufl.constantvalue import Zero
         if isinstance(summand, Zero):
             sh = summand.shape()
-            j, = index
+
+            if isinstance(index, Index):
+                j = index
+            elif isinstance(index, MultiIndex):
+                if len(index) != 1:
+                    error("Expecting a single Index only.")
+                j, = index
+
             fi = tuple(i for i in summand.free_indices() if not i == j)
             idims = dict(summand.index_dimensions())
             del idims[j]
@@ -43,9 +54,7 @@ class IndexSum(AlgebraOperator):
 
     def __init__(self, summand, index):
         AlgebraOperator.__init__(self)
-        if not isinstance(summand, Expr):
-            error("Expecting Expr instance, not %s." % repr(summand))
-        
+
         if isinstance(index, Index):
             j = index
         elif isinstance(index, MultiIndex):
@@ -59,11 +68,12 @@ class IndexSum(AlgebraOperator):
 
         d = self._index_dimensions[j]
         self._index = as_multi_index(index, (d,))
+        ufl_assert(isinstance(self._index, MultiIndex), "Error in initialization of index sum.")
         self._dimension = d
         del self._index_dimensions[j]
 
-        self._repr = "IndexSum(%r, %r)" % (summand, index)
-    
+        self._repr = "IndexSum(%r, %r)" % (self._summand, self._index)
+
     def index(self):
         return self._index[0]
     
@@ -81,7 +91,11 @@ class IndexSum(AlgebraOperator):
     
     def shape(self):
         return self._summand.shape()
-    
+
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return self._summand.is_cellwise_constant()
+
     def evaluate(self, x, mapping, component, index_values):
         i, = self._index
         tmp = 0
