@@ -11,14 +11,14 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-06-19
-# Last changed: 2009-06-19
+# Last changed: 2011-06-02
 
 from ufl.log import error
 from ufl.assertions import ufl_assert
@@ -35,7 +35,11 @@ class LiftingResult(Operator):
         ufl_assert(isinstance(operand, Expr), "Expecting an Expr.")
         self._operator = operator
         self._operand = operand
-        self._shape = jump(self._operand, self._operator.cell().n).shape() # FIXME: Is this right?
+
+        # FIXME: Compute shape without making jump so we don't need the cell here:
+        cell = self._operator.cell()
+        n = cell.n
+        self._shape = jump(self._operand, n).shape() # FIXME: Is this right?
 
     def operands(self):
         return (self._operator, self._operand)
