@@ -12,16 +12,16 @@ output messages. These may be redirected by the user of UFL."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Johan Hake, 2009.
 #
 # First added:  2005-02-04
-# Last changed: 2011-04-12
+# Last changed: 2011-06-08
 
 import sys
 import types
@@ -30,7 +30,8 @@ import logging
 log_functions = ["log", "debug", "info", "warning", "error", "begin", "end",
                  "set_level", "push_level", "pop_level", "set_indent", "add_indent",
                  "set_handler", "get_handler", "get_logger", "add_logfile", "set_prefix",
-                 "info_red", "info_green", "info_blue"]
+                 "info_red", "info_green", "info_blue",
+                 "warning_red", "warning_green", "warning_blue"]
 
 __all__ = log_functions + ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL", "Logger", "log_functions"]
 
@@ -129,6 +130,18 @@ class Logger:
         "Write warning message."
         self._log.warning(self._format(*message))
 
+    def warning_red(self, *message):
+        "Write warning message in red."
+        self._log.warning(RED % self._format(*message))
+
+    def warning_green(self, *message):
+        "Write warning message in green."
+        self._log.warning(GREEN % self._format(*message))
+
+    def warning_blue(self, *message):
+        "Write warning message in blue."
+        self._log.warning(BLUE % self._format(*message))
+
     def error(self, *message):
         "Write error message and raise an exception."
         self._log.error(*message)
