@@ -12,16 +12,16 @@ and generating index lists."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Martin Alnes 2009
 #
 # First added:  2008-05-22
-# Last changed: 2009-01-09
+# Last changed: 2011-06-08
 
 def compute_indices(shape):
     "Compute all index combinations for given shape"
@@ -39,6 +39,29 @@ def compute_indices2(shape):
     "Compute all index combinations for given shape"
     return ((),) if len(shape) == 0 else tuple((i,) + sub_index for i in xrange(shape[0]) for sub_index in compute_indices2(shape[1:]))
 
+def build_component_numbering(shape, symmetry):
+    """Build a numbering of components within the given value shape,
+    taking into consideration a symmetry mapping which leaves the
+    mapping noncontiguous. Returns a dict { component -> numbering }
+    and an ordered list of components [ numbering -> component ].
+    The dict contains all components while the list only contains
+    the ones not mapped by the symmetry mapping."""
+    vi2si, si2vi = {}, []
+    indices = compute_indices(shape)
+    # Number components not in symmetry mapping
+    for c in indices:
+        if c not in symmetry:
+            vi2si[c] = len(si2vi)
+            si2vi.append(c)
+    # Copy numbering to mapped components
+    for c in indices:
+        if c in symmetry:
+            vi2si[c] = vi2si[symmetry[c]]
+    # Validate
+    for k, c in enumerate(si2vi):
+        assert vi2si[c] == k
+    return vi2si, si2vi
+
 def compute_permutations(k, n, skip = None):
     """Compute all permutations of k elements from (0, n) in rising order.
     Any elements that are contained in the list skip are not included."""
