@@ -12,16 +12,16 @@ for all types that are terminal nodes in the expression trees."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008
 #
 # First added:  2008-03-14
-# Last changed: 2008-03-27
+# Last changed: 2011-08-08
 
 from ufl.expr import Expr, WrapperType
 from ufl.common import lstr
@@ -56,7 +56,6 @@ class Terminal(Expr):
     def evaluate(self, x, mapping, component, index_values, derivatives=()):
         "Get self from mapping and return the component asked for."
         f = mapping.get(self)
-        
         # No mapping, trying to evaluate self as a constant
         if f is None:
             try:
@@ -65,13 +64,17 @@ class Terminal(Expr):
                     f = 0.0
                 return f
             except:
-                warning("Couldn't map '%s' to a float, returning object unchanged." % str(self))
+                pass
+            # If it has an ufl_evaluate function, call it
+            if hasattr(self, 'ufl_evaluate'):
+                return self.ufl_evaluate(x, component, derivatives)
             # Take component if any
+            warning("Couldn't map '%s' to a float, returning ufl object without evaluation." % str(self))
             f = self
             if component:
                 f = f[component]
             return f
-        
+
         # Found a callable in the mapping
         if callable(f):
             if derivatives:
@@ -119,6 +122,9 @@ class UtilityType(Terminal):
     
     def index_dimensions(self):
         error("Calling index_dimensions on a utility type is an error.")
+    
+    def is_cellwise_constant(self):
+        error("Calling is_cellwise_constant on a utility type is an error.")
 
 #--- Non-tensor terminal nodes ---
 
@@ -126,7 +132,7 @@ class Tuple(WrapperType):
     "For internal use, never to be created by users."
     __slots__ = ("_items", "_repr")
     def __init__(self, *items):
-        Expr.__init__(self)
+        WrapperType.__init__(self)
         if not all(isinstance(i, Expr) for i in items):
             error("Got non-Expr in Tuple, is this intended? If so, remove this error.")
         self._items = tuple(items)
@@ -159,3 +165,22 @@ class Tuple(WrapperType):
     def __repr__(self):
         return self._repr
 
+class Data(UtilityType):
+    "For internal use, never to be created by users."
+    __slots__ = ("_data", "_repr")
+    def __init__(self, data):
+        UtilityType.__init__(self)
+        self._data = data
+        self._repr = "Data(%r)" % (self._data,)
+
+    def reconstruct(self):
+        return self
+
+    def operands(self):
+        return ()
+
+    def __str__(self):
+        return "Data(%s)" % str(self._data)
+
+    def __repr__(self):
+        return self._repr
