@@ -6,15 +6,15 @@ cell = triangle
 element = FiniteElement("CG", cell, 1)
 v = TestFunction(element)
 u = TrialFunction(element)
-f = Function(element)
+f = Coefficient(element)
 
 velement = VectorElement("CG", cell, 1)
 vv = TestFunction(velement)
 vu = TrialFunction(velement)
-vf = Function(velement)
+vf = Coefficient(velement)
 
 telement = TensorElement("CG", cell, 1)
 tv = TestFunction(telement)
 tu = TrialFunction(telement)
-tf = Function(telement)
+tf = Coefficient(telement)
 
