@@ -12,14 +12,14 @@ expressions as variables for differentiation."""
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2008-05-20
-# Last changed: 2009-04-27
+# Last changed: 2011-06-02
 
 from ufl.common import Counted
 from ufl.log import error
@@ -85,6 +85,9 @@ class Variable(WrapperType):
     def cell(self):
         return self._expression.cell()
 
+    def is_cellwise_constant(self):
+        return self._expression.is_cellwise_constant()
+
     def evaluate(self, x, mapping, component, index_values):
         a = self._expression.evaluate(x, mapping, component, index_values)
         return a
